package DLMListener;

import MoveNClick.ClickEvent;
import MoveNClick.Event;
import MoveNClick.MoveEvent;
import MoveNClick.MoveNClick;
import MoveNClick.PressEvent;
import MoveNClick.ReleaseEvent;
import MoveNClick.ZoomEvent;
import TouchRecognizer.Point;
import java.awt.Component;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**
 *
 * @author dlm
 */
public class EventDispatcher extends Thread
{
    // *** ATTRIBUTI ***
    private Vector<Event> eventi;                           // vettore degli eventi
    private Hashtable<Component, DLMListener> ascoltatori;  // hashtable dei componenti-ascoltatori
    private MoveNClick moveNClick;                          // istanza del notificatori di eventi
    private static EventDispatcher istance;                 // istanza dell'oggetto dispatcher

    // *** METODI ***
    private EventDispatcher()
    {
		try
		{
			moveNClick = MoveNClick.getIstance();
			ascoltatori = new Hashtable<Component, DLMListener>();
			this.start();
		}
		catch(NoClassDefFoundError ex)
		{
			ascoltatori = null;
			System.out.println("Touch non avviato: " + ex);
		}
    }

    /**
     * Metodo ricorsivo per ottenere la posizione assoluta di un componente sullo schermo
     * @param c Componente da analizzare
     * @return La posizione assoluta del componente analizzato
     */
    private Point trovaPosition(Component c)
    {
        Point posizione;
        // se il componente attuale è di root
        // valuta la distanza dai bordi dello schermo
        if(c.getParent() == null)
            posizione = new Point(c.getX(), c.getY());
        // altrimenti valuta la distanza dai bordi del padre
        else
        {
            // effettua la ricorsione sulla profondità dell'albero dei componenti
            Point posPadre = trovaPosition(c.getParent());

            posizione = new Point(c.getX() + posPadre.getX(), c.getY() + posPadre.getY());
        }

        return posizione;
    }

    /**
     * Metodo ricorsivo che calcola l'altezza del componente sull'albero dei componenti Swing
     * @param c Il componente da valutare
     * @return  L'altezza del componente sull'albero dei componenti Swing
     */
    private int numParent(Component c)
    {
        int depth = 1;

        // se il componente è di root la profondità è 1
        if(c.getParent() == null)
            depth = 1;
        else
            // altrimenti va in ricorsione
            depth += numParent(c.getParent());

        return depth;
     }
    
    @Override
    public void run()
    {
        while(true)
        {
            Vector<ComponentEvents> componentiInteressati = new Vector<ComponentEvents>();

            // acquisisce gli eventi dal notificatore degli eventi
            this.eventi = moveNClick.getEvents();

            // scorre il vettore degli eventi e guarda la posizione in cui è avvenuto l'evento
            for(int i = 0; i < eventi.size(); i++)
            {
                Event evento = eventi.get(i);
                int x = evento.getX();
                int y = evento.getY();

                Component max = null;
                int depth = 0;

                // scorre l'hash table per trovare il componente a cui passare l'evento
                for (Enumeration e = ascoltatori.keys(); e.hasMoreElements();)
                {
                   Component c = (Component) e.nextElement();

                   // controlla se il componente è visibile
                   if(c.isVisible())
                   {
                       // controlla se è più profondo del componente finora candidao per la notifica
                       if(numParent(c) >= depth)
                       {
                           // e controlla se è nella posizione dell'evento
                           Point posComp = trovaPosition(c);
                           if(x >= posComp.getX() && x <= (posComp.getX() + c.getWidth()) &&
                              y >= posComp.getY() && y <= (posComp.getY() + c.getHeight()))
                           {

                               // allora assegna il componente all'evento
                                max = c;
                                depth = numParent(c);
                                evento.setSource(c);
                           }
                       }
                   }

                }

                // controlla se esiste un componente candidato per l'evento
                if(max != null)
                {
                    // se esiste inserisce l'evento nel vettore degli eventi per il componente candidato
                    // così da controllare gli eventi per un componente in un secondo momento, in modo
                    // da poter valutare la presenza di eventi abbinabili

                    // se non è presente un contenitore per il componente candidato
                    // ne crea uno in coda
                   if(!componentiInteressati.contains(max))
                       componentiInteressati.add(new ComponentEvents(max));

                    // aggiunge l'evento al componente appropriato
                    for(int j = 0; j < componentiInteressati.size(); j++)
                    {
                        if(componentiInteressati.elementAt(j).equals(max))
                            componentiInteressati.elementAt(j).addEvent(evento);
                    }

                }
            }

            // scorro il vettore dei contenitori e scatena gli eventi sul listener appropriato
            for(ComponentEvents tmp: componentiInteressati)
            {
                // variabili utilizzate per discriminare la presenza di zoom sul componente
                MoveEvent move1 = null,
                          move2 = null;
                PressEvent press = null;

                // ottiene il listener associato al componente
                DLMListener listener = ascoltatori.get(tmp.getComponent());

                // scorre gli eventi e propaga immediatamente solo click e release
                // in quanto i move e press possono essere abbinati fra loro per creare
                // un evento di tipo Zoom
                for(Event evento : tmp.getEvents())
                {
                    // se l'evento è di tipo press lo immagazzina
                    // in previsione di un possibile zoom
                    if(evento instanceof PressEvent)
                    {
                        if(press == null)
                            press = (PressEvent) evento;
                        else
                            listener.pressEvent((PressEvent) evento);
                    }
                    // propaga un evento di tipo click
                    else if(evento instanceof ClickEvent)
                        listener.clickEvent((ClickEvent) evento);
                    // propaga un evento di tipo release
                    else if(evento instanceof ReleaseEvent)
                        listener.releaseEvent((ReleaseEvent) evento);
                    // se l'evento è di tipo move li immagazzina
                    // in previsione di un possibile zoom
                    else if(evento instanceof MoveEvent)
                    {
                        if(move1 == null)
                            move1 = (MoveEvent) evento;
                        else if(move2 == null)
                            move2 = (MoveEvent) evento;
                        // se sono già presenti due move abbinabili per uno zoom
                        // i restanti move vengono scatenati al listener
                        else
                            listener.moveEvent((MoveEvent)evento);
                    }
                }

                // Possibili combinazioni per la creazione di zoom
                // Combinazione 1: Move1-Move2
                // Combinazione 2: Press-Move1
                if(move1 != null)
                {
                    if(move2 == null)
                    {
                        // c'è solo un move
                        if(press == null)
                        {
                            // quindi non può essere abbinato per uno zoom
                            // dunque deve essere scatenato come move
                            listener.moveEvent(move1);
                        }
                        // se c'è un move e un press
                        else
                        {
                            // crea un evento di tipo zoom secondo la Combinazione 2
                            ZoomEvent zoom = new ZoomEvent(press, move1);
                            listener.zoomEvent(zoom);
                        }
                    }
                    // ci sono due move
                    else
                    {
                        // crea un evento di tipo zoom secondo la Combinazione 1
                        ZoomEvent zoom = new ZoomEvent(move1, move2);
                        listener.zoomEvent(zoom);
                        // quindi il press può essere scatenato in maniera indipendente
                        listener.pressEvent(press);
                    }
                }
                // se non esistono eventi move da abbinare all'evento press immagazzinato
                // allora scatena il press in maniera indipendente
                if(move1 == null && move2 == null && press != null)
                    // diamo il press
                    listener.pressEvent(press);
            }
        }
    }

    /**
     * Aggiunge il DLMListener al componente specificato
     * @param c Componente da monitorare
     * @param listener Ascoltatore da utilizzare per la notifica degli eventi
     */
    public void addListener(Component c, DLMListener listener)
    {
		if(ascoltatori != null)
			ascoltatori.put(c, listener);
    }

	/**
     * Rimuove il DLMListener dal componente specificato
     * @param c Componente di cui rimuovere l'ascoltatore
     */
	public void removeListener(Component c)
	{
		if(ascoltatori != null)
			ascoltatori.remove(c);
	}

    /**
     * Ottiene l'istanza al dispatcher
     * @return L'istanza del dispatcher
     */
    public static EventDispatcher getIstance()
    {
        if(istance == null)
            istance = new EventDispatcher();

        return istance;
    }

    /*
     * Classe innestata per la gestione dell'algoritmo
     */
    private class ComponentEvents
    {
        private Component component;
        private Vector<Event> eventi;

        /**
         * Crea un contenitore che associa ad ogni componente gli eventi scatenati su di esso
         * @param component Il componente proprietario degli eventi
         */
        public ComponentEvents(Component component)
        {
            this.component = component;
            this.eventi = new Vector<Event>();
        }

        /**
         * Ottiene il componente proprietario degli eventi
         * @return Il componente proprietario degli eventi
         */
        public Component getComponent()
        {
            return component;
        }

        /*
         * Aggiunge l'evento nel contenitore associato al componente
         * @param e L'evento da aggiungere
         */
        public void addEvent(Event e)
        {
            this.eventi.add(e);
        }

        /*
         * Ottiene il vettore degli eventi relativi al componente
         * @return Il vettore degli eventi relativi al componente
         */
        public Vector<Event> getEvents()
        {
            return eventi;
        }

        @Override
        public boolean equals(Object obj)
        {
            boolean esito = false;

            if(obj instanceof Component)
            {
                Component c = (Component) obj;

                if(c == this.component)
                    esito = true;
            }

            return esito;
        }

        @Override
        public int hashCode() {
            int hash = 7;
            hash = 41 * hash + (this.component != null ? this.component.hashCode() : 0);
            return hash;
        }
    }
}


