package TouchRecognizer;

import java.awt.BorderLayout;
import java.awt.Component;
import java.io.IOException;
import javax.media.Buffer;
import javax.media.CannotRealizeException;
import javax.media.Manager;
import javax.media.MediaLocator;
import javax.media.NoPlayerException;
import javax.media.Player;
import javax.media.control.FrameGrabbingControl;
import javax.media.format.VideoFormat;
import javax.swing.JFrame;

/**
 *
 * @author dlm
 */
public class CamManager extends JFrame
{
    private static CamManager istance = null;
    private Player player;
    private MediaLocator mediaLocator;
    private Component comp;
    private int width, height;

    private CamManager() throws NoPlayerException, CannotRealizeException, IOException
    {
        // inizializzo le variabili
        player = null;
        mediaLocator = null;
        comp = null;
        width = 0;
        height = 0;

        settaggioPlayer();
		if(player != null)
		{
			setVisualComponents();
			setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			setVisible(true);

			// effettua un polling  temporizzato
			// in attesa che la webcam sia disponibile
			while(player.getState() != player.Started)
			{
				try
				{
					Thread.sleep(500);
				}
				catch(InterruptedException ex)
				{
					System.out.println(ex);
				}
			}
		}
    }

    /**
     * Acquisisce ed avvia lo stream della webcam
     */
    private void settaggioPlayer() throws NoPlayerException, CannotRealizeException, IOException
    {
        // inizializza il media locator sul bus della webcam
		this.mediaLocator = new MediaLocator("vfw://0");
		this.player = Manager.createRealizedPlayer(mediaLocator);
		// avvia l'acquisizione dello stream
		this.player.start();
    }

    /**
     * Abbina la webcam al jFrame
     */
    private void setVisualComponents()
    {
        comp = player.getVisualComponent();
        add(comp, BorderLayout.NORTH);
        this.pack();
    }

    /**
     * Ottiene il canale Red (RGB) dell'immagine catturata dalla webcam. Se l'immagine
     * non è pronta il canale sarà null
     * @return l'array di byte contenente il canale Red se l'immagine è pronta, null altrimenti
     */
    public byte[] getRawData()
    {
        byte[] image = null;
        // prende l'immagine dal player
        FrameGrabbingControl fbc = (FrameGrabbingControl) player.getControl("javax.media.control.FrameGrabbingControl");
        Buffer buffer = fbc.grabFrame();

        // se sono presenti dati nel buffer della webcam
        // li valida
        if(buffer != null && buffer.getData() != null)
        {
           if(buffer.getData() instanceof byte[])
           {
               if(buffer.getFormat() != null)
               {
                   // dal formato della webcam caratterizza
                   // i parametri di larghezza ed altezza
                   VideoFormat format = (VideoFormat) buffer.getFormat();
                   this.width = format.getSize().width;
                   this.height = format.getSize().height;
               }

               // effettua il cast ad array di byte
               // per poter ottenere solo il canale red
               // dell'immagine
               byte[] rawData = (byte[]) buffer.getData();

               image = new byte[rawData.length / 3];

               // carica nel nuovo array solo il canale Red
               for(int i = 0; i < image.length; i++)
                   image[i] = rawData[i*3];
           }
        }

        return image;
    }

    /**
     * Ritorna la larghezza dell'immagine
     * @return la larghezza dell'immagine
     */
    public int getImageWidth()
    {
        return width;
    }

    /**
     * Ritorna l'altezza dell'immagine
     * @return l'altezza dell'immagine
     */
    public int getImageHeight()
    {
        return height;
    }

    /**
     * Ottiene l'istanza della webcam secondo il pattern singleton
     * @return l'istanza della webcam
     */
    public synchronized static CamManager getIstance() throws NoPlayerException, CannotRealizeException, IOException
    {
        if(istance == null)
            istance = new CamManager();

        return istance;
    }
}
