package TouchRecognizer;

/**
 *
 * @author dlm
 */
public class Point
{
    // *** ATTRIBUTI ***
    private int x, y;                     // Posizione del punto
    private int xOrigin, yOrigin;
    private int spostamentoAssoluto;
    // soglie
    private static int sameThreshold = 1; // Soglia sotto della quale due punti non sono distinguibili

    // *** METODI ***
    /**
     * Crea un nuovo punto nella posizione specificata
     * @param x ascissa del punto
     * @param y ordinata del punto
     */
    public Point(int x, int y)
    {
        // Setto gli attributi
        this.xOrigin = this.x = x;
        this.yOrigin = this.y = y;
        this.spostamentoAssoluto = 0;
    }

    /**
     * Ritorna la distanza rispetto al punto specificato
     * @param p Punto di su cui effettuare il confronto
     * @return Distanza rispetto al punto specificato
     */
    public double distance(Point p)
    {
        return Math.sqrt((double)(x - p.x) * (x - p.x) +    // quadrato cateto orizzontale
                            (double) (y - p.y) * (y - p.y));// quadrato cateto verticale
    }

     /**
     * Restituisce l'asscissa di origine del punto
     * @return L'ascissa di origine del punto
     */
    public int getOriginX()
    {
        return this.xOrigin;
    }

    /**
     * Restituisce l'ordinata di origine del punto
     * @return L'ordinata di origine del punto
     */
    public int getOriginY()
    {
        return this.yOrigin;
    }
    
    /**
     * Restituisce lo spostamento assoluto effettuato dal punto
     * @return Lo spostamento assoluto effettuato dal punto
     */
    public int getSpostamentoAssoluto()
    {
        return spostamentoAssoluto;
    }
    
    /**
     * Restituisce l'asscissa del punto
     * @return L'ascissa del punto
     */
    public int getX()
    {
        return this.x;
    }

    /**
     * Restituisce l'ordinata del punto
     * @return L'ordinata del punto
     */
    public int getY()
    {
        return this.y;
    }

    /**
     * Aggiorno il punto alla nuova posizione ed eventualmente nel caso di spostamento elevato
     * aggiorna lo spostamento assoluto
     * @param x Ascissa del punto di destinazione
     * @param y Ordinata del punto di destinazione
     */
    public void sposta(int x, int y)
    {
        if(!this.isTheSame(new Point(x,y)))
           spostamentoAssoluto += distance(new Point(x,y));
        
        this.x = x;
        this.y = y;
    }

    /**
     * Aggiorno il punto alla nuova posizione ed eventualmente nel caso di spostamento elevato
     * aggiorna lo spostamento assoluto
     * @param p Punto di destinazione
     */
    public void sposta(Point p)
    {
        sposta(p.getX(), p.getY());
    }

    /**
     * Valuta se due punti in posizioni diverse possono essere ricondotti allo stesso punto
     * @param p Punto su cui effettuare il confronto
     * @return L'esito della valutazione
     */
    public boolean isTheSame(Point p)
    {
        return ((new Point(this.xOrigin, this.yOrigin).distance(p)) <= sameThreshold) && spostamentoAssoluto == 0;
    }

    /**
     * Setta la soglia di confronto di similarità tra punti
     * @param threshold Soglia di similarità
     */
    public static void setSameThreshold(int threshold)
    {
        sameThreshold = threshold;
    }
    
    /**
     * Confronta due punti e dice se hanno le stesse coordinate
     * @param obj Oggetto su cui effettuare il confronto
     * @return l'esito booleano del confronto
     */
    @Override
    public boolean equals(Object obj)
    {
        boolean esito = false;

        if(obj instanceof Point)
        {
            Point tmp = (Point) obj;

            esito = this.x == tmp.x && this.y == tmp.y;
        }

        return esito;
    }

    /**
     * Ritorna l'hash del punto
     * @return L'hash del punto
     */
    @Override
    public int hashCode()
    {
        int hash = 7;
        hash = 23 * hash + this.x;
        hash = 23 * hash + this.y;
        return hash;
    }

    /**
     * Restituisce il punto in formato stringa
     * @return Rappresentazione del punto in formato stringa
     */
    @Override
    public String toString()
    {
        return "(" + this.x + "," + this.y + ")";
    }
}