package TouchRecognizer;

/**
 *
 * @author dlm
 */
public class Shape
{
    // *** ATTRIBUTI ***
    private boolean firstPixel;
    private boolean[][] shape;
    private int area;
    private double eccentricita;

    // *** METODI ***
    /**
     * Costruisce una nuova forma che potrà avere al massimo le dimensioni specificate.
     * @param imageWidth Larghezza dell'immagine di origine.
     * @param imageHeight Altezza dell'immagine di origine.
     */
    public Shape(int imageWidth, int imageHeight)
    {
        area = 0;
        eccentricita = 0;
        firstPixel = true;
        // Inizializza la matrice di ricerca della forma
        shape = new boolean[imageWidth][imageHeight];
        for(int i = 0; i < imageWidth; i++)
                for(int j = 0; j < imageHeight; j++)
                        shape[i][j] = false;
    }

    /**
     * Ottiene il baricentro della forma
     * @return Il baricentro della forma
     */
    public Point getBarycenter()
    {
        int x = 0;
        int y = 0;
        int numPoint = 0;
        int maxX = 0, maxY = 0, minX = shape.length, minY = shape[0].length;

        // Somma ascisse e ordinate e incrementa il numero di punti
        for(int i = 0; i < shape.length; i++)
            for(int j = 0; j < shape[i].length; j++)
                    if(shape[i][j])
                    {
                        if(i > maxX)
                            maxX = i;
                        if(i < minX)
                            minX = i;
                        if(j > maxY)
                            maxY = j;
                        if(j < minY)
                            minY = j;

                        x += i;
                        y += j;
                        numPoint++;
                    }

        // calcola eccentricità (aggiunge 1 per considerare lo spessore del pixel)
        this.eccentricita = (maxX - minX + 1) / (double) (maxY - minY + 1);

        // dividento infine per il numero di punti
        // ottiene il baricentro della forma
        x = x / numPoint;
        y = y / numPoint;

        return new Point(x,y);
    }

    /**
     * Restituisce l'area della forma
     * @return L'area della forma
     */
    public int getArea()
    {
        return area;
    }

    /**
     * Valuta se la forma riconosciuta è circolare
     * @return L'eccentricità della forma
     */
    public boolean isRound()
    {
        // ritorna true se l'eccentricità è vicina a 1 (che identifica una figura circolare)
        return (this.eccentricita < 2 && this.eccentricita > 0.5);
    }

    /**
     * Cerca nell'intorno del punto se esistono punti appartenenti alla figura.
     * Se esistono aggiunge anche questo punto alla forma
     * @param p Il punto da aggiungere alla forma
     * @return Vero se il punto è stato aggiunto alla forma, falso altrimenti
     */
    public boolean tryAttachPoint(Point p)
    {
        // cerca se ci sono punti nell'intorno
        // oppure se e' il primo punto della forma
        if(firstPixel ||
            (p.getX() > 0 && shape[p.getX() -1][p.getY()]) ||                       // sinistra
            (p.getX() < shape.length - 1 && shape[p.getX() +1][p.getY()]) ||        // destra
            (p.getY() > 0 && shape[p.getX()][p.getY() -1]) ||                       // sopra
            (p.getY() < shape[p.getX()].length - 1 && shape[p.getX()][p.getY()+1])) // sotto
        {
            shape[p.getX()][p.getY()] = true;
            firstPixel = false;
            area++;
        }

        return shape[p.getX()][p.getY()];
    }

    /**
     * Ottiene la matrice bidimensionale booleana rappresentante la forma
     * @return La matrice bidimensionale booleana rappresentante la forma
     */
    public boolean[][] getShape()
    {
        return shape;
    }
}
